Package: M3Drop
Version: 1.39.0
Date: 2024-03-25
Title: Michaelis-Menten Modelling of Dropouts in single-cell RNASeq
Author: Tallulah Andrews <tallulandrews@gmail.com>
Maintainer: Tallulah Andrews <tallulandrews@gmail.com>
Depends: R (>= 3.4), numDeriv
Imports: RColorBrewer, gplots, bbmle, statmod, grDevices, graphics,
        stats, matrixStats, Matrix, irlba, reldist, Hmisc, methods,
        scater
Suggests: ROCR, knitr, M3DExampleData, SingleCellExperiment, Seurat,
        Biobase
VignetteBuilder: knitr
biocViews: RNASeq, Sequencing, Transcriptomics, GeneExpression,
        Software, DifferentialExpression, DimensionReduction,
        FeatureExtraction
Collate: basics.R Plotting_fxns.R Curve_fitting.R Extremes.R
        Normalization.R Brennecke_implementation.R
        Threeway_ProportionalArea_VennDiagrams.R
        Simulations_Functions.R Traditional_DE.R NB_UMI.R
        M3D_Imputation.R Other_FS_functions.R DANB_HVG.R
        DANB_Coexpression.R
Description: This package fits a model to the pattern of dropouts in
        single-cell RNASeq data. This model is used as a null to
        identify significantly variable (i.e. differentially expressed)
        genes for use in downstream analysis, such as clustering cells.
        Also includes an method for calculating exact Pearson residuals
        in UMI-tagged data using a library-size aware negative binomial
        model.
URL: https://github.com/tallulandrews/M3Drop
BugReports: https://github.com/tallulandrews/M3Drop/issues
License: GPL (>=2)
Config/pak/sysreqs: libcairo2-dev cmake libfontconfig1-dev
        libfreetype6-dev libfribidi-dev make libharfbuzz-dev libicu-dev
        libjpeg-dev libpng-dev libtiff-dev libuv1-dev libwebp-dev
        zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:44:35 UTC
RemoteUrl: https://github.com/bioc/M3Drop
RemoteRef: HEAD
RemoteSha: 987a662dbbe2c91ea3e79db1ca43457b46c15de6
NeedsCompilation: no
Packaged: 2026-05-30 09:37:51 UTC; root
Built: R 4.6.0; ; 2026-05-30 13:37:49 UTC; unix
