exportPattern("M3Drop*")
exportPattern("NBumi*")
export(corFS)
export(giniFS)
export(irlbaPcaFS)
export(Consensus_FS)
export(BrenneckeGetVariableGenes)
export(PoissonUMIFeatureSelectionDropouts)
import(RColorBrewer)
import(gplots)
import(bbmle)
import(statmod)
import(irlba)
import(Matrix)
import(numDeriv)
importFrom("grDevices", "col2rgb", "colorRampPalette", "densCols")
importFrom("Hmisc", "rcorr")
importFrom("graphics", "abline", "axis", "box", "curve", "frame", "hist", "legend", "lines", "mtext", "par", "points", "text", "title", "polygon")
importFrom("stats", "aggregate", "dist", "dnorm", "fitted", "glm", "hclust", "lm", "median", "p.adjust", "pchisq", "pnorm", "quantile", "rnorm", "sd", "uniroot", "var", "wilcox.test", "cor", "dnbinom", "optim", "runif", "rbinom", "prcomp", "density", "rgamma", "aov", "loess", "rnbinom", "qnbinom", "pnbinom", "qnorm")
importFrom("reldist", "gini")
importFrom("matrixStats", "rowSds")
importFrom("matrixStats", "rowVars")
importFrom("matrixStats", "colVars")
importFrom("methods", "as")
